@@ -1815,123 +1815,18 @@ meta_prefs_set_num_workspaces (int n_workspaces)
 #endif /* HAVE_GCONF */
 }
 
-/* Indexes must correspond to MetaKeybindingAction */
+#define keybind(name, handler, param, flags, stroke, description) \
+  { #name, NULL, flags & BINDING_REVERSES },
 static MetaKeyPref screen_bindings[] = {
-  { META_KEYBINDING_WORKSPACE_1, NULL, FALSE },
-  { META_KEYBINDING_WORKSPACE_2, NULL, FALSE },
-  { META_KEYBINDING_WORKSPACE_3, NULL, FALSE },
-  { META_KEYBINDING_WORKSPACE_4, NULL, FALSE },
-  { META_KEYBINDING_WORKSPACE_5, NULL, FALSE },
-  { META_KEYBINDING_WORKSPACE_6, NULL, FALSE },
-  { META_KEYBINDING_WORKSPACE_7, NULL, FALSE },
-  { META_KEYBINDING_WORKSPACE_8, NULL, FALSE }, 
-  { META_KEYBINDING_WORKSPACE_9, NULL, FALSE },
-  { META_KEYBINDING_WORKSPACE_10, NULL, FALSE },
-  { META_KEYBINDING_WORKSPACE_11, NULL, FALSE },
-  { META_KEYBINDING_WORKSPACE_12, NULL, FALSE },
-  { META_KEYBINDING_WORKSPACE_LEFT, NULL, FALSE },
-  { META_KEYBINDING_WORKSPACE_RIGHT, NULL, FALSE },
-  { META_KEYBINDING_WORKSPACE_UP, NULL, FALSE },
-  { META_KEYBINDING_WORKSPACE_DOWN, NULL, FALSE },
-  { META_KEYBINDING_SWITCH_GROUP, NULL, TRUE },
-  { META_KEYBINDING_SWITCH_GROUP_BACKWARD, NULL, TRUE },
-  { META_KEYBINDING_SWITCH_WINDOWS, NULL, TRUE },
-  { META_KEYBINDING_SWITCH_WINDOWS_BACKWARD, NULL, TRUE },
-  { META_KEYBINDING_SWITCH_PANELS, NULL, TRUE },
-  { META_KEYBINDING_SWITCH_PANELS_BACKWARD, NULL, TRUE },
-  { META_KEYBINDING_CYCLE_GROUP, NULL, TRUE },
-  { META_KEYBINDING_CYCLE_GROUP_BACKWARD, NULL, TRUE },
-  { META_KEYBINDING_CYCLE_WINDOWS, NULL, TRUE },
-  { META_KEYBINDING_CYCLE_WINDOWS_BACKWARD, NULL, TRUE },
-  { META_KEYBINDING_CYCLE_PANELS, NULL, TRUE },
-  { META_KEYBINDING_CYCLE_PANELS_BACKWARD, NULL, TRUE },
-  { META_KEYBINDING_SHOW_DESKTOP, NULL, FALSE },
-  { META_KEYBINDING_PANEL_MAIN_MENU, NULL, FALSE },
-  { META_KEYBINDING_PANEL_RUN_DIALOG, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_1, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_2, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_3, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_4, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_5, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_6, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_7, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_8, NULL, FALSE }, 
-  { META_KEYBINDING_COMMAND_9, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_10, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_11, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_12, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_13, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_14, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_15, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_16, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_17, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_18, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_19, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_20, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_21, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_22, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_23, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_24, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_25, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_26, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_27, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_28, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_29, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_30, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_31, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_32, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_SCREENSHOT, NULL, FALSE },
-  { META_KEYBINDING_COMMAND_WIN_SCREENSHOT, NULL, FALSE },
-  { META_KEYBINDING_RUN_COMMAND_TERMINAL, NULL, FALSE },
-  { META_KEYBINDING_SET_SPEW_MARK, NULL, FALSE },
+#include "screen-bindings.h"
   { NULL, NULL, FALSE}
 };
 
 static MetaKeyPref window_bindings[] = {
-  { META_KEYBINDING_WINDOW_MENU, NULL, FALSE },
-  { META_KEYBINDING_TOGGLE_FULLSCREEN, NULL, FALSE },
-  { META_KEYBINDING_TOGGLE_MAXIMIZE, NULL, FALSE },
-  { META_KEYBINDING_TOGGLE_ABOVE, NULL, FALSE },
-  { META_KEYBINDING_MAXIMIZE, NULL, FALSE },
-  { META_KEYBINDING_UNMAXIMIZE, NULL, FALSE },
-  { META_KEYBINDING_TOGGLE_SHADE, NULL, FALSE },
-  { META_KEYBINDING_MINIMIZE, NULL, FALSE },
-  { META_KEYBINDING_CLOSE, NULL, FALSE },
-  { META_KEYBINDING_BEGIN_MOVE, NULL, FALSE },
-  { META_KEYBINDING_BEGIN_RESIZE, NULL, FALSE },
-  { META_KEYBINDING_TOGGLE_STICKY, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_1, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_2, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_3, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_4, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_5, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_6, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_7, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_8, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_9, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_10, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_11, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_12, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_LEFT, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_RIGHT, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_UP, NULL, FALSE },
-  { META_KEYBINDING_MOVE_WORKSPACE_DOWN, NULL, FALSE },
-  { META_KEYBINDING_RAISE_OR_LOWER, NULL, FALSE },
-  { META_KEYBINDING_RAISE, NULL, FALSE },
-  { META_KEYBINDING_LOWER, NULL, FALSE },
-  { META_KEYBINDING_MAXIMIZE_VERTICALLY, NULL, FALSE },
-  { META_KEYBINDING_MAXIMIZE_HORIZONTALLY, NULL, FALSE },
-  { META_KEYBINDING_MOVE_TO_CORNER_NW, NULL, FALSE },
-  { META_KEYBINDING_MOVE_TO_CORNER_NE, NULL, FALSE },
-  { META_KEYBINDING_MOVE_TO_CORNER_SW, NULL, FALSE },
-  { META_KEYBINDING_MOVE_TO_CORNER_SE, NULL, FALSE },
-  { META_KEYBINDING_MOVE_TO_SIDE_N, NULL, FALSE },
-  { META_KEYBINDING_MOVE_TO_SIDE_S, NULL, FALSE },
-  { META_KEYBINDING_MOVE_TO_SIDE_E, NULL, FALSE },
-  { META_KEYBINDING_MOVE_TO_SIDE_W, NULL, FALSE },
-  { META_KEYBINDING_MOVE_TO_CENTER, NULL, FALSE },
+#include "window-bindings.h"
   { NULL, NULL, FALSE }
 };
+#undef keybind
 
 #ifndef HAVE_GCONF
 typedef struct
@@ -1940,44 +1835,28 @@ typedef struct
   const char *keybinding;
 } MetaSimpleKeyMapping;
 
-/* Name field must occur in the same order as screen_bindings, though entries
- * can be skipped
+/* FIXME: This would be neater if the array only contained entries whose
+ * default keystroke was non-null.  You COULD do this by defining
+ * ONLY_BOUND_BY_DEFAULT around various blocks at the cost of making
+ * the bindings file way more complicated.  However, we could stop this being
+ * data and move it into code.  Then the compiler would optimise away
+ * the problem lines.
  */
+
+#define keybind(name, handler, param, flags, stroke, description) \
+  { #name, keystroke },
+
 static MetaSimpleKeyMapping screen_string_bindings[] = {
-  { META_KEYBINDING_WORKSPACE_LEFT,         "<Control><Alt>Left"         },
-  { META_KEYBINDING_WORKSPACE_RIGHT,        "<Control><Alt>Right"        },
-  { META_KEYBINDING_WORKSPACE_UP,           "<Control><Alt>Up"           },
-  { META_KEYBINDING_WORKSPACE_DOWN,         "<Control><Alt>Down"         },
-  { META_KEYBINDING_SWITCH_WINDOWS,         "<Alt>Tab"                   },
-  { META_KEYBINDING_SWITCH_PANELS,          "<Control><Alt>Tab"          },
-  { META_KEYBINDING_CYCLE_GROUP,            "<Alt>F6"                    },
-  { META_KEYBINDING_CYCLE_WINDOWS,          "<Alt>Escape"                },
-  { META_KEYBINDING_CYCLE_PANELS,           "<Control><Alt>Escape"       },
-  { META_KEYBINDING_SHOW_DESKTOP,           "<Control><Alt>d"            },
-  { META_KEYBINDING_PANEL_MAIN_MENU,        "<Alt>F1"                    },
-  { META_KEYBINDING_PANEL_RUN_DIALOG,       "<Alt>F2"                    },
-  { META_KEYBINDING_COMMAND_SCREENSHOT,     "Print"                      },
-  { META_KEYBINDING_COMMAND_WIN_SCREENSHOT, "<Alt>Print"                 },
+#include "screen-bindings.h"
   { NULL,                                   NULL                         }
 };
 
-/* Name field must occur in the same order as window_bindings, though entries
- * can be skipped
- */
 static MetaSimpleKeyMapping window_string_bindings[] = {
-  { META_KEYBINDING_WINDOW_MENU,            "<Alt>Print"                 },
-  { META_KEYBINDING_MAXIMIZE,               "<Alt>F10"                   },
-  { META_KEYBINDING_UNMAXIMIZE,             "<Alt>F5"                    },
-  { META_KEYBINDING_MINIMIZE,               "<Alt>F9"                    },
-  { META_KEYBINDING_CLOSE,                  "<Alt>F4"                    },
-  { META_KEYBINDING_BEGIN_MOVE,             "<Alt>F7"                    },
-  { META_KEYBINDING_BEGIN_RESIZE,           "<Alt>F8"                    },
-  { META_KEYBINDING_MOVE_WORKSPACE_LEFT,    "<Control><Shift><Alt>Left"  },
-  { META_KEYBINDING_MOVE_WORKSPACE_RIGHT,   "<Control><Shift><Alt>Right" },
-  { META_KEYBINDING_MOVE_WORKSPACE_UP,      "<Control><Shift><Alt>Up"    },
-  { META_KEYBINDING_MOVE_WORKSPACE_DOWN,    "<Control><Shift><Alt>Down"  },
+#include "window-bindings.h"
   { NULL,                                   NULL                         }
 };
+#undef keybind
+
 #endif /* NOT HAVE_GCONF */
 
 static void
@@ -2075,6 +1954,9 @@ init_bindings (void)
   int which = 0;
   while (window_string_bindings[i].name)
     {
+      if (window_string_bindings[i].keybinding == NULL)
+        continue;
+
       /* Find which window_bindings entry this window_string_bindings entry
        * corresponds to.
        */
@@ -2093,6 +1975,9 @@ init_bindings (void)
   which = 0;
   while (screen_string_bindings[i].name)
     {
+      if (screen_string_bindings[i].keybinding == NULL)
+        continue;
+    
       /* Find which window_bindings entry this window_string_bindings entry
        * corresponds to.
        */
@@ -2949,6 +2834,7 @@ meta_prefs_get_compositing_manager (void)
 void
 meta_prefs_set_compositing_manager (gboolean whether)
 {
+#ifdef HAVE_GCONF
   GError *err = NULL;
 
   gconf_client_set_bool (default_client,
@@ -2962,6 +2848,9 @@ meta_prefs_set_compositing_manager (gboolean whether)
                     err->message);
       g_error_free (err);
     }
+#else
+  compositing_manager = whether;
+#endif
 }
 
 #ifndef HAVE_GCONF
